# Import local modules
from photoshop._core import Photoshop


class PhotoshopSaveOptions(Photoshop):

    def __int__(self):
        super().__init__()
        # If true, the alpha channels are saved.
        self.AlphaChannels = False
        # If true, the annotations are save
        self.AnnotationsProperty = False
        self.Layers = True



"""Check whether the foreground is equal to the background color.

References:
    https://github.com/lohriialo/photoshop-scripting-python/blob/master/CompareColors.py

"""
import photoshop as ps

app = ps.Application()

if app.foregroundColor.isEqual(app.backgroundColor):
    print("They're Equal")
else:
    print("NOT Equal")



try:
    # python-2.
    import _winreg as winreg
except ImportError:
    # python-3
    import winreg
import os

from comtypes import COMError
from comtypes.client import CreateObject
from photoshop import constants
from photoshop.errors import PhotoshopPythonAPIError


class Photoshop(object):
    _root = 'Photoshop'
    REG_PATH = 'Software\\Adobe\\Photoshop'
    _object_name = 'Application'
    object_name = None

    def __init__(self, ps_version=None, parent=None):
        self._progress_id = None
        version_mappings = constants.PHOTOSHOP_VERSION_MAPPINGS
        self.photoshop_version = os.getenv('PS_VERSION', ps_version)
        version = self._get_install_version()
        self.app_id = version_mappings.get(self.photoshop_version, version)
        try:
            self.app = self.instance_app(self.app_id)
        except OSError:
            try:
                self.app = self.instance_app(self._get_install_version())
            except OSError:
                raise PhotoshopPythonAPIError(
                    'Please check if you have '
                    'Photoshop installed correctly.',
                )
        if parent:
            self.adobe = self.app
            self.app = parent

    @property
    def typename(self):
        return self.__class__.__name__

    def __call__(self, *args, **kwargs):
        return self.app

    def __str__(self):
        return f'{self.__class__.__name__} <{self._progress_id}>'

    def __repr__(self):
        return self

    def __getattribute__(self, item):
        try:
            return super().__getattribute__(item)
        except AttributeError:
            return getattr(self.app, item)

    def get_application_path(self):
        key = winreg.OpenKey(
            winreg.HKEY_LOCAL_MACHINE,
            f'{self.REG_PATH}\\{self.app_id}',
        )
        return winreg.QueryValueEx(key, 'ApplicationPath')[0] + 'Photoshop'

    def instance_app(self, ps_id):
        naming_space = [self._root]
        if not self.object_name:
            naming_space.append(self._object_name)
        else:
            naming_space.append(self.object_name)
        naming_space.append(ps_id)
        self._progress_id = self._get_name(naming_space)
        return self._create_object(self._progress_id, dynamic=True)

    def _get_install_version(self):
        key = winreg.OpenKey(
            winreg.HKEY_LOCAL_MACHINE,
            self.REG_PATH,
        )
        self.app_id = winreg.EnumKey(key, 0).split('.')[0]
        return self.app_id

    @staticmethod
    def _create_object(*args, **kwargs):
        return CreateObject(*args, **kwargs)

    @staticmethod
    def _get_name(list_):
        return '.'.join(list_)

    def eval_javascript(self, javascript, Arguments=None, ExecutionMode=None):
        return self.adobe.doJavaScript(javascript, Arguments, ExecutionMode)



try:
    from photoshop.action_descriptor import ActionDescriptor
    from photoshop.action_refrence import ActionReference
    from photoshop.application import Application
    from photoshop.artlayer import ArtLayer
    from photoshop.layerSets import LayerSets
    from photoshop.layerSet import LayerSet
    from photoshop.colors import (
        LabColor,
        HSBColor,
        CMYKColor,
        RGBColor,
    )
    from photoshop.document import Document
    from photoshop.documents import Documents
    from photoshop.layer import Layer
    from photoshop.save_options import (
        GIFSaveOptions,
        JPEGSaveOptions,
        PDFSaveOptions,
        PNGSaveOptions,
    )
    from photoshop.solid_color import SolidColor
    from photoshop.text_item import TextItem
    from photoshop.enumerations import NoiseDistribution
    from photoshop.enumerations import Units
    from photoshop.enumerations import LayerKind
    from photoshop.enumerations import NewDocumentMode
    from photoshop.enumerations import DocumentFill
    from photoshop.enumerations import DialogModes
    from photoshop.enumerations import SelectionType
    from photoshop.enumerations import TextureType
    from photoshop.enumerations import ColorBlendMode
    from photoshop.enumerations import StrokeLocation
    from photoshop.constants import *
except ModuleNotFoundError:
    # Fix Build docs failed on readthedocs.
    pass

# All public APIs.
__all__ = [
    'ActionDescriptor',
    'ActionReference',
    'ArtLayer',
    'Application',
    'CMYKColor',
    'ColorBlendMode',
    'DialogModes',
    'DocumentFill',
    'Document',
    'Documents',
    'SelectionType',
    'StrokeLocation',
    'TextureType',
    'NewDocumentMode',
    'LayerKind',
    'SolidColor',
    'TextItem',
    'LabColor',
    'LayerSets',
    'LayerSet',
    'HSBColor',
    'RGBColor',
    'Layer',
    'NoiseDistribution',
    'JPEGSaveOptions',
    'PNGSaveOptions',
    'PDFSaveOptions',
    'GIFSaveOptions',
    'Units',
]

__title__ = 'photoshop_python_api'
__version__ = '0.4.0'
__author__ = 'Long Hao'
__license__ = 'MIT'
__copyright__ = 'Copyright 2018 Long Hao'



from photoshop._core import Photoshop


class Selection(Photoshop):
    def __init__(self, parent=None):
        super().__init__(parent=parent)

    @property
    def bounds(self):
        return self.app.bounds

    def parent(self):
        return self.app.parent

    @property
    def solid(self):
        return self.app.solid

    @property
    def typename(self):
        return self.app.typename

    def clear(self):
        self.app.clear()

    def contract(self):
        self.app.contract()

    def copy(self):
        self.app.copy()

    def cut(self):
        self.app.cut()

    def select(self, *args, **kwargs):
        return self.app.select(*args, **kwargs)

    def deselect(self):
        return self.app.deselect()

    def expand(self, by):
        """Expands the selection.

        Args:
            by (int): The amount to expand the selection.

        """
        self.app.expand(by)

    def feather(self, by):
        """Feathers the edges of the selection.

        Args:
            by (int): The amount to feather the edge.

        """
        return self.app.feather(by)

    def fill(self, fillType, mode, opacity, preserveTransparency):
        """Fills the selection.

        Args:
            fillType (str): The color or history state with which to fill the
                object.
            mode (): The color blend mode.
            opacity (int): The opacity as a percentage. Range: 1 to 100.
            preserveTransparency (bool): If true, perserves transparencies.

        """
        return self.app.fill(fillType, mode, opacity, preserveTransparency)

    def grow(self, tolerance, antiAlias):
        """Grows the selection to include all adjacent pixels falling within

        the specified tolerance range.

        Args:
            tolerance (int): The tolerance range. Range: 0 to 255.
            antiAlias (bool): If true, anti-aliasing is used.


        """
        return self.app.grow(tolerance, antiAlias)

    def stroke(self, strokeColor, width, location, mode, opacity,
               preserveTransparency):
        """Strokes the selection.

        Args:
            strokeColor (SolidColor): The color to stroke the selection with.
            width (int): The stroke width.
            location (int): The stroke location.
            mode (int): The color blend mode.
            opacity (int): The opacity of the stroke color as a percentage.
                Range: 1 to 100.
            preserveTransparency (bool): If true, preserves transparency.

        """
        return self.app.stroke(strokeColor, width, location, mode, opacity,
                               preserveTransparency)

    def selectBorder(self, width):
        """Selects the selection border only (in the specified width);
        subsequent actions do not affect the selected area within the borders.

        Args:
            width (int): The width of the border selection.

        """
        return self.app.selectBorder(width)



# Set the active layer to the last art layer of the active document, or the
# first if the last is already active.


import photoshop as ps

app = ps.Application()
if app.documents.length < 1:
    docRef = app.documents.add()
else:
    docRef = app.activeDocument

if docRef.layers.length < 2:
    docRef.artLayers.add()


print(docRef.activeLayer.name)
print(docRef.layers.item(docRef.layers.length))
# Set current active to first layer.
# docRef.activeLayer = docRef.layers.item(1)
new_layer = docRef.artLayers.add()
print(new_layer.name)
new_layer.name = "test"




import photoshop as ps

MAX_THUMB_SIZE = 1280
app = ps.Application()
doc = app.activeDocument
orig_name = doc.name
width_str = doc.width
height_str = doc.height
thumb_name = f'{orig_name}_tumb'
index = width_str / MAX_THUMB_SIZE

thumb_width = int(width_str / index)

thumb_height = int(height_str / index)

thumb_doc = doc.duplicate(thumb_name)
thumb_doc.resizeImage(thumb_width, thumb_height - 100)
o = ps.JPEGSaveOptions()
thumb_doc.saveAs('d:/thumb.jpg', o, asCopy=True)
thumb_doc.close()



from photoshop._core import Photoshop
from photoshop.artlayer import ArtLayer
from photoshop.layer import Layer


class ArtLayers(Photoshop):

    def __init__(self, parent):
        super().__init__(parent=parent)

    @property
    def _layers(self):
        return [layer for layer in self.app]

    def __iter__(self):
        for layer in self.app:
            yield layer

    def __getitem__(self, item):
        return self.app[item]

    @property
    def length(self):
        return len(self._layers)

    @property
    def parent(self):
        return self.app.parent

    @property
    def typename(self):
        return self.app.typename

    def add(self):
        """Adds an element."""
        return Layer(self.app.add())

    def getByName(self, name):
        return self.app.getByName(name)

    def removeAll(self):
        return self.app.removeAll()

    def link(self, layer):
        self.app.link(layer)
